% Identify seismic global spatio-temporal teleconnections with a Self-Organizing Map

% Load dataset for period of time 
% 1973/01/01 - 2011/06/14, i.e. matrix 35603x8.
load neic_epic_global.txt;

s = neic_epic_global';

% Seismic data are de-noised as images. Function wpdencmp is used for seismic image blind de-noising. 
% Find default values (ddencmp).
[thr,sorh,keepapp,crit] = ddencmp('den','wp',s);

% Biorthogonal spline bases were found to have the most optimal correction and de-noising capabilities. 
% De-noise and correct seismic image data using global thresholding with SURE the most optimal basis.
sd = wpdencmp(s,sorh,3,'bior6.8',crit,thr,keepapp);

% Residual errors are estimated to be very small.
re = s - sd;


inputs = sd;

% Create a Self-Organizing Map: 8 x 900 x 900.
dimension1 = 30;
dimension2 = 30;
net = selforgmap([dimension1 dimension2]);

% Train the Network. Training takes about 9.7 hours for dimensions 30, for 1000 epochs. 
% Accuracy of identified seismic teleconnections is very high.    
net.trainParam.epochs = 1000;
%net.trainParam.goal = 0.001;
[net,tr] = train(net,inputs);

% Test the Network.
outputs = net(inputs);

% View the Network.
view(net)

% Plot seismic global spatio-temporal teleconnections.
figure, 
plotsomtop(net)

figure,
plotsomnc(net)

figure, 
plotsomnd(net)

figure, 
plotsomplanes(net)

figure, 
plotsomhits(net,inputs)

figure, 
plotsompos(net,inputs)


% The same values in row vector variable (clusters) indicate seismic
% teleconnection in original USGS NEIC epic global database 35603x8.
y = net(inputs);
clusters = vec2ind(y);






